package controller;

import simulator.*;

/**
 * The control logic for the PhoneDrone.
 *
 * @author CS159 Professors, James Madison University
 * @version 1.0
 */
public class FoneDroneController extends DroneController {

    private int[] location;

    /**
     * Explicit Value Constructor.
     *
     * @param number The phone number to look for
     */
    public FoneDroneController(String number) {
        super(number);
        this.location = null;
    }

    /**
     * Get the location of the phone (or null if it hasn't been found).
     * 
     * @return The location of the phone (or null)
     */
    public int[] getLocationOfPhone() {
        return location;
    }

    /**
     * Start the FoneDrone.
     */
    @Override
    public void start() {
        search();
    }
    

    /**
     * Search (recursively).
     */
    public void search() {
        if (location == null)
            location = drone.checkForPhone(number);

        // The base case (there is no direction to search in)
        if (!shouldLook(Direction.FORWARD) && !shouldLook(Direction.RIGHT)
                && !shouldLook(Direction.BACKWARD)
                && !shouldLook(Direction.LEFT))
            return;

        
        // Refinement
        if (shouldLook(Direction.FORWARD)) {
            drone.move(Direction.FORWARD);
            search();
            drone.move(Direction.BACKWARD);
        }

        if (shouldLook(Direction.RIGHT)) {
            drone.move(Direction.RIGHT);
            search();
            drone.move(Direction.LEFT);
        }

        if (shouldLook(Direction.BACKWARD)) {
            drone.move(Direction.BACKWARD);
            search();
            drone.move(Direction.FORWARD);
        }

        if (shouldLook(Direction.LEFT)) {
            drone.move(Direction.LEFT);
            search();
            drone.move(Direction.RIGHT);
        }
    }

    /**
     * Should the Drone look in the given direction?
     * 
     * This method can incorporate whether the Drone can move in the given
     * Direction, whether it has been in that direction or not, and
     * whether the phone has already been found or not.
     * 
     * @param d The Direction of interest
     * @return true if it should look; false otherwise
     */
    private boolean shouldLook(Direction d) {
        return (location == null) && drone.canMove(d) && !drone.hasBeen(d);
    }

}
